unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, TeeGDIPlus, TeEngine, ExtCtrls, TeeProcs, Chart, Series, TeeTools,
  StdCtrls, Math, EditChar;

type
  TIntegerArray = array of Integer;
  TDoubleArray = array of Double;

  TXYPoints = class
    XValues: TDoubleArray;
    YValues: TDoubleArray;
  end;

  TForm1 = class(TForm)
    Chart1: TChart;
    Panel1: TPanel;
    GBOptions: TGroupBox;
    CBPointers: TCheckBox;
    CBSegments: TCheckBox;
    CBInterpolate: TCheckBox;
    Button1: TButton;
    procedure FormCreate(Sender: TObject);
    procedure CBPointersClick(Sender: TObject);
    procedure CBSegmentsClick(Sender: TObject);
    procedure CBInterpolateClick(Sender: TObject);
    procedure Button1Click(Sender: TObject);
  private
    { Private declarations }
    function InterpGetPointerStyle(Sender:TChartSeries; ValueIndex:Integer): TSeriesPointerStyle;
    procedure checkColors;
    function appendDouble(value: Double; toArray: TDoubleArray): TDoubleArray;
    procedure recalcInterpolations;
    procedure ColorLineDrag(Sender:TColorLineTool);
    function SearchCrossingPoints(Values: TChartValues; ACount: Integer; YValue: double): TIntegerArray;
    function InterpolateSeries(Series: TChartSeries; YValue: Double): TDoubleArray;
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

var sourceLine, interpLine: TLineSeries;
    colorLine: TColorLineTool;
    interpPoints: TDoubleArray;

procedure TForm1.FormCreate(Sender: TObject);
begin
  Chart1.View3D:=false;
  Chart1.Legend.Visible:=false;

  sourceLine:=Chart1.AddSeries(TLineSeries) as TLineSeries;
  with sourceLine do
  begin
    Pointer.Visible:=true;
    ColorEachLine:=true;

    FillSampleValues;
  end;

  interpLine:=CloneChartSeries(sourceLine) as TLineSeries;

  interpLine.OnGetPointerStyle:=InterpGetPointerStyle;

  sourceLine.Visible:=false;

  colorLine:=Chart1.Tools.Add(TColorLineTool) as TColorLineTool;
  with colorLine do
  begin
    Axis:=Chart1.Axes.Left;
    Value:=sourceLine.YValues.MinValue + (sourceLine.YValues.MaxValue - sourceLine.YValues.MinValue) / 2;
    OnDragLine:=ColorLineDrag;
  end;

  CBInterpolate.Enabled:=CBSegments.Checked;
  checkColors;
end;

procedure TForm1.checkColors;
var pointerColor, segmentColor: TColor;
    i: Integer;
begin
  Chart1.AutoRepaint:=false;

  recalcInterpolations;

  if CBPointers.Checked then
    pointerColor:=clRed
  else
    pointerColor:=sourceLine.Color;

  if CBSegments.Checked then
    segmentColor:=clRed
  else
    segmentColor:=sourceLine.Color;

  for i:=0 to interpLine.Count-1 do
    if (colorLine<>nil) then
    begin
      interpLine.Pointer.Items[i].Color:=sourceLine.Color;
      interpLine.ValueColor[i]:=sourceLine.Color;

      if interpLine.YValue[i]<colorLine.Value then
      begin
        interpLine.Pointer.Items[i].Color:=pointerColor;
        interpLine.ValueColor[i]:=segmentColor;
      end
      else
      if interpLine.YValue[i]=colorLine.Value then
      begin
        if (i-1>-1) and (interpLine.YValue[i-1]<=colorLine.Value) then
        begin
          interpLine.Pointer.Items[i].Color:=pointerColor;
          interpLine.ValueColor[i]:=segmentColor;
        end;
      end;
    end;

  Chart1.AutoRepaint:=true;
  Chart1.Draw;
end;

function TForm1.InterpGetPointerStyle(Sender:TChartSeries; ValueIndex:Integer): TSeriesPointerStyle;
var i: Integer;
begin
  if (ValueIndex>-1) and (ValueIndex<Sender.Count) then
    if (Sender.YValue[ValueIndex] = colorLine.Value) then
      result:=psNothing
    else
      result:=(Sender as TLineSeries).Pointer.Style;
end;

procedure TForm1.recalcInterpolations;
var i, idx: Integer;
begin
  interpLine.Clear;
  interpLine.DataSource:=sourceLine;
  interpLine.CheckDataSource;

  for i:=0 to interpLine.Count-1 do
    interpLine.Pointer.Items[i].Visible:=true;

  if CBSegments.Checked and CBInterpolate.Checked then
  begin
    interpPoints:=InterpolateSeries(sourceLine, colorLine.Value);

    for i:=0 to High(interpPoints) do
      interpLine.AddXY(interpPoints[i], colorLine.Value);
  end;

end;

function TForm1.appendDouble(value: Double; toArray: TDoubleArray): TDoubleArray;
var i: Integer;
begin
  SetLength(result, length(toArray)+1);
  for i:=0 to High(toArray) do
    result[i]:=toArray[i];

  result[length(toArray)]:=value;
end;

function TForm1.InterpolateSeries(Series: TChartSeries; YValue: Double): TDoubleArray;
var
  crossIndex: TIntegerArray;
  i: Integer;
  dx, dy: Double;
begin
  SetLength(result, 0);

  crossIndex:=SearchCrossingPoints(Series.YValues.Value, Series.Count, YValue);

  for i:=0 to High(crossIndex) do
  begin
    // safeguard
    if (crossIndex[i] < 1) then crossIndex[i] := 1
    else if (crossIndex[i] >= Series.Count) then crossIndex[i]:=Series.Count-1;

    // x = (y-y1)*(x2-x1)/(y2-y1)+x1
    dx := Series.XValues.Value[crossIndex[i]] - Series.XValues.Value[crossIndex[i] - 1];
    dy := Series.YValues.Value[crossIndex[i]] - Series.YValues.Value[crossIndex[i] - 1];

    if (dx <> 0) then
      Result:=appendDouble(((YValue - Series.YValues.Value[crossIndex[i] - 1]) * dx / dy) + Series.XValues.Value[crossIndex[i] - 1], result);
  end;
end;

function TForm1.SearchCrossingPoints(Values: TChartValues; ACount: Integer; YValue: double): TIntegerArray;

  function appendInt(value: Integer; toArray: TIntegerArray): TIntegerArray;
  var i: Integer;
  begin
    SetLength(result, length(toArray)+1);
    for i:=0 to High(toArray) do
      result[i]:=toArray[i];

    result[length(toArray)]:=value;
  end;

var i: Integer;
begin
  SetLength(result, 0);

  for i:=0 to ACount-2 do
  begin
    if ((Values[i]>YValue) and (Values[i+1]<YValue)) or
       ((Values[i]<YValue) and (Values[i+1]>YValue)) then

      result:=appendInt(i+1, result);
  end;
end;

procedure TForm1.CBPointersClick(Sender: TObject);
begin
  checkColors;
end;

procedure TForm1.CBSegmentsClick(Sender: TObject);
begin
  CBInterpolate.Enabled:=CBSegments.Checked;

  checkColors;
end;

procedure TForm1.Button1Click(Sender: TObject);
begin
  EditChart(Self, Chart1);
end;

procedure TForm1.CBInterpolateClick(Sender: TObject);
begin
  checkColors;
end;

procedure TForm1.ColorLineDrag(Sender:TColorLineTool);
begin
  checkColors;
end;

end.
